//
//	NetworkStat.js
//	Widgets
/*

Copyright 2005/2006, Pascal Pfiffner.

*/


// Globals
// default-settings: 'whilehidden', 7200, false

var seconds = getTimer();			// seconds left till next refresh
var refreshMode = getRefreshMode();			// refresh-mode
var refreshOnShow = getRefreshOnShow();				// refresh on every Dashboard-activation?
var mail_notification_on = getMailNotificationOn();			// send mails on new IP?
var stylesheet = getStylesheet();
var dateformat = getDateformat();

var pingIP = getPingIP();					// get the IP the user wants to ping

var doCountdown = (refreshMode == 'manual') ? 0 : 1;		// countdown active or not?
var lastRefresh = new Date();
var currentLocation = "unknown";
var the_interval;
var iface_count = 0;					// how many interfaces have assigned IPs
var iparr = new Array();				// holds the current IPs (index is the "iface_count" of the corresponding interface)
var no_network_tries = 0;				// if no network is present, raise this one and retry after 10, 20 or 30 seconds. If after 3 tries no network is there, give up.

var _version = 0;

var recent_ips = getRecentIPs();



// Dashboard Setup by Apple (sets the stylesheet, creates the "done" button and refreshes for the first time)
function setup() {
	setStylesheet(stylesheet);		// set stylesheet matching prefs
	document.getElementById('refreshbutton').src = "Images_" + stylesheet + "/refresh.png";	// set refreshbutton matching stylesheet
	createGenericButton(document.getElementById("done"), "done", savePrefs);
	
	switchPrefTab("refresh");
	_version = getCurrentVersion();
	
	// refresh state
	setTimeout("refreshState()", 100);
}




/*********************
 *	Main Functions   *
 *********************/
function refreshState() {		// refresh info
	setTimer("in progress...");
	window.clearInterval(the_interval);
	var no_net = false;
	
	if(window.widget) {
		
		// Location
		setupLocations();
		document.getElementById("change_location_cover").src = "Images_" + stylesheet + "/Home.png";
		
		purgeInterfaces();
		
		// ******
		// Internal
		addParagraph('main', "None", "checking...");	// add the checking-indicator
		adjustTotalSize();								// adjusts widgetsize
		var internals = widget.system("./get_internal.pl", null).outputString;
		var interfaces = internals.split("\n");
		purgeLastChild("maincontent");
		
		iface_count = 0;
		iparr = Array();
		for(i = 0; i < interfaces.length; i++) {
			if(interfaces[i].length > 0) {
				
				// arrange the array
				var iface = new Array();
				var iface_arr = new Array();
				var parts = interfaces[i].split("|");
				for(var p = 0; p < parts.length; p++) {
					var foo = parts[p].split("?");
					if(foo[1]) {
						iface[foo[0]] = foo[1];
						iface_arr.push(new Array(foo[0], foo[1]));
					}
				}
				
				// the interface-name
				addParagraph('main', iface['name'], iface, 0);
				addParagraph('iface_more', 0, iface_arr, "interface_more");
				iparr[iface_count] = iface['ipv4'];
				iface_count++;
			}
		}
		
		// ******
		// External
		addParagraph('main', "Internet", "checking...", 0);		// adds another checking-indicator
		adjustTotalSize();								// the checking-indicator should be inside the widget
		var external = widget.system("/usr/bin/curl --connect-timeout 8 -s www.whatismyip.org", null).outputString;
		
		if(external) {
			
			// check for errors
			if((external.toString()).length > 15) {
				if(external.search("requests received from your IP address") > 1) {
					external = 'too frequent';
				}
				else {
					external = 'error';
				}
			}

			// save IP (unixstamp:ip)
			else {
				external = external.replace(/[^\d\.]+/g, '');
				saveAndNotifyThisIP(external);
			}
			
			purgeLastChild("maincontent");
			addParagraph('main', "Internet", external, 0);
			
			// Ping
			addParagraph('wan_more', 0, 0, "interface_more");
			//toggleMoreInfo(document.getElementById("maincontent").lastChild.previousSibling, event);
			setTimeout("startJSPing()", 100);
		}
		else {
			no_net = true;
			external = "no network";
			purgeLastChild("maincontent");
			if( ! (iface_count == 1 && iparr[0] == external))		// only add "no network" if the one single internal interface has no Network, too
				addParagraph('main', "Internet", external, 0);
		}
	}
	
	
	// No Widget
	else {
		displayLocation("still unknown");
		
		purgeInterfaces();
		addParagraph('main', "None", "no dashboard", 0);
	}
	
	adjustTotalSize();
	setTimer("done");		// set the timer (countdown will reset it in a second, but lets do that anyway)
	
	lastRefresh = new Date();
	document.getElementById("lastRefresh").innerHTML = formatDate(lastRefresh);
	
	// recheck if there was no WAN
	if(no_net) {
		no_network_tries++;
		
		if(no_network_tries <= 3) {
			seconds = no_network_tries * 10;
			countdown();
			return;
		}
	}
	
	no_network_tries = 0;
	
	seconds = getTimer();
	countdown();
}


// calls the shell script
var pingResult;
function startJSPing() {
	if(window.widget) {
		if(!pingIP)
			pingIP = "4.2.2.2";
		
		document.getElementById("ping_packets").innerHTML = "ping in progress...";
		try {
			pingResult = widget.system("./ping.sh " + pingIP, displayJSPing);
		}
		catch(exc) {
			alert("error while pinging");
		}
	}
	
	return false;
}

// gets the result from startJSPing and displays it
function displayJSPing() {
	if(pingResult.outputString && document.getElementById("ping_packets")) {
		var arr = pingResult.outputString.replace(/[^\d\. \/]/, "").split(/\/| /);
		var sent = arr[0] ? arr[0] : 0;
		var rec = arr[1] ? arr[1] : 0;
		
		document.getElementById("ping_packets").innerHTML = Math.round((rec / sent) * 100) + "% of " + sent + " packets received";
		document.getElementById("ping_min").innerHTML = arr[2] ? arr[2] : "&#8734;";
		document.getElementById("ping_avg").innerHTML = arr[3] ? arr[3] : "&#8734;";
		document.getElementById("ping_max").innerHTML = arr[4] ? arr[4] : "&#8734;";
	}
}


function countdown() {		// display time left to refresh and initiate refresh on t=0
	if(doCountdown && (refreshMode != 'manual')) {
		if(seconds > 3600) {
			if(seconds % 3600 == 0)
				setTimer("in " + (seconds/3600) + " hours");
			else {
				var hours = Math.floor(seconds / 3600);
				var minutes = Math.floor((seconds % 3600) / 60);
				setTimer("in " + hours + ((hours == 1) ? " hour" : " hours") + " " + minutes + ((minutes == 1) ? " minute" : " minutes"));
			}
			
			seconds -= 60;
			
			window.clearInterval(the_interval);
			the_interval = window.setInterval("countdown()", 60000);
		}
		
		else if(seconds > 60) {
			setTimer("in about " + Math.ceil(seconds / 60) + " minutes");
			
			seconds -= 60;
			if(seconds < 60)
				seconds = 60;
			
			window.clearInterval(the_interval);
			the_interval = window.setInterval("countdown()", 60000);
		}
		
		else if(seconds > 0) {
			setTimer("in " + seconds + ((seconds == 1) ? " second" : " seconds"));
			seconds--;
			
			window.clearInterval(the_interval);
			the_interval = window.setInterval("countdown()", 996);
		}
		
		else {
			refreshState();
		}
	}
	
	
	else {
		window.clearInterval(the_interval);
		setTimer("MANUALLY");
	}
}


function setTimer(text) {
	document.getElementById('timer').innerHTML = text;
}


function displayLocation() {
	var obj = document.getElementById("location");
	var current = currentLocation ? currentLocation : "unknown";
	
	obj.innerHTML = current;
}

function setupLocations() {
	var locations = new Array();
	if(window.widget) {
		var locationstring = widget.system("./get_locations.pl", null).outputString;
		locations = locationstring.split(/\n/g);
	}
	else {
		locations.push("0	not available");
	}
	
	// empty old locations
	var sel = document.getElementById("change_location_menu");
	while(null != sel.lastChild)
		sel.removeChild(sel.lastChild);
	
	// loop all locations
	for(var i = 0; i < locations.length; i++) {
		// get location info
		var values = locations[i].split(/\t/);
		var selected = (values[0] == 1);
		var value = values[1];
		var name = values[2];
		
		// set up an option-element
		var opt = document.createElement("option");
		opt.value = value;
		opt.innerHTML = name;
		sel.appendChild(opt);
		
		// current location
		if(selected) {
			opt.selected = selected;
			currentLocation = name;
		}
	}
	
	// add option to open Network Prefs and Network Doctor
	var opt1 = document.createElement("option");
	opt1.value = -1;
	opt1.innerHTML = "----";
	sel.appendChild(opt1);
	var opt2 = document.createElement("option");
	opt2.value = -2;
	opt2.innerHTML = "Open Network Preferences";
	sel.appendChild(opt2);
	var opt3 = document.createElement("option");
	opt3.value = -3;
	opt3.innerHTML = "Open Network Diagnostics";
	sel.appendChild(opt3);
	
	displayLocation();
}

// called when a new location is chosen from the drop-down
function changeLocation(id, name) {
	no_network_tries = 0;
	
	if(window.widget) {
		if(id == -2) {
			widget.openApplication("com.apple.systempreferences");
			return;
		}
		else if(id == -3) {
			widget.openApplication("com.apple.NetworkDiagnostics");
			return;
		}
		else if(id != -1) {
			var set_success = widget.system("./set_location.pl " + id, null).outputString;
			currentLocation = set_success;
		}
	}
	
	setTimeout("refreshState()", 50);
}


function purgeInterfaces() {
	var parent = document.getElementById('maincontent');
	var elements = parent.childNodes;
	var a = elements.length;
	
	for(i = a; i > 0; i--) {
		if(elements[i] && elements[i].nodeName == 'P')
			parent.removeChild(elements[i]);
	}
}

function purgeLastChild(id) {
	var parent = document.getElementById(id);
	parent.removeChild(parent.lastChild);
}


// adds a paragraph, supply a type to know which one we must add
function addParagraph(type, iface_name, data_arr, my_class) {
	var parent = document.getElementById('maincontent');
	var data = (typeof(data_arr) == "object") ? data_arr : new Array(data_arr);
	
	// <p>
	var paragraph = document.createElement("p");
	if(my_class)
		paragraph.setAttribute("class", my_class);
			
	
	// we have a major paragraph (that is, an interface main paragraph with image)
	if('main' == type) {
		
		// Interface-Image (or text if interface is not known)
		if(iface_name) {
			var img;
			if('unknown' == iface_name) {
				img = document.createElement("span");
				img.setAttribute("class", "interface_unknownname");
				img.innerHTML = data_arr['ifname'] ? data_arr['ifname'] : "??";
			}
			else {
				img = document.createElement("img");
				img.setAttribute("src", "Images_" + stylesheet + "/" + iface_name + ".png");
				img.setAttribute("class", "interface_img");
			}
			
			img.setAttribute("onclick", "toggleMoreInfo(this.parentNode, event)");
			paragraph.appendChild(img);
		}
		
		paragraph.appendChild(getClipboardLink(data['ipv4'] ? data['ipv4'] : data[0], "data_big"));
	}
	
	// we have an interface-moreinfo-paragraph
	else if('iface_more' == type) {
		var iface = new Array();
		for(var i = 0; i < data_arr.length; i++) {
			var key = data_arr[i][0];
			var val = data_arr[i][1];
			//alert(key + " => " + val);
			iface[key] = val;
		}
		
		// Compose main Info; IPv6 and Prefixlength
		paragraph.appendChild(getClipboardLink(iface['ipv6']));
		paragraph.appendChild(document.createTextNode(" | "));
		paragraph.appendChild(getClipboardLink(iface['prefixlen']));
		
		// Gateway
		if(iface['server_identifier'] || iface['router']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("Gateway", "inline_div_3"));
			paragraph.appendChild(getDiv(getClipboardLink(iface['server_identifier']), "inline_div_6"));
		}
		
		// Netmask
		if(iface['subnet_mask'] || iface['netmask']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("Netmask", "inline_div_3"));
			if(iface['subnet_mask']) {
				paragraph.appendChild(getDiv(getClipboardLink(iface['subnet_mask']), "inline_div_6"));
			}
			else {
				paragraph.appendChild(getDiv(getClipboardLink(iface['netmask']), "inline_div_6"));
			}
		}
		
		// Broadcast
		if(iface['bcast']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("Broadcast", "inline_div_3"));
			paragraph.appendChild(getDiv(getClipboardLink(iface['bcast']), "inline_div_6"));
		}
		
		// SSID (for wireless networks
		if(iface['ssid']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("SSID", "inline_div_3"));
			paragraph.appendChild(getDiv(getClipboardLink(iface['ssid']), "inline_div_6"));
		}
		
		// Wireless channel
		if(iface['wireless_channel']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("Channel", "inline_div_3"));
			paragraph.appendChild(getDiv(getClipboardLink(iface['wireless_channel']), "inline_div_6"));
		}
		
		// MAC
		if(iface['MAC']) {
			paragraph.appendChild(document.createElement("br"));
			paragraph.appendChild(getDiv("MAC", "inline_div_3"));
			paragraph.appendChild(getDiv(getClipboardLink(iface['MAC']), "inline_div_6"));
		}
		
		// DNS Servers
		if(iface['dns']) {
			var dns_arr = iface['dns'].split(",");
			
			for(var i = 0; i < dns_arr.length; i++) {
				paragraph.appendChild(document.createElement("br"));
				paragraph.appendChild(getDiv("DNS " + (i + 1), "inline_div_3"));
				paragraph.appendChild(getDiv(getClipboardLink(dns_arr[i]), "inline_div_6"));
			}
		}
	}
			
	// the WAN-moreinfo-paragraph
	else if('wan_more' == type) {
		
		// the last IPs
		if(recent_ips.length > 0) {
			paragraph.appendChild(getDiv("Your last WAN-IPs", "title"));
			
			// current IP (time info)
			var thisdate = new Date(recent_ips[0]['time'] * 1000);
			var datestring = formatDate(thisdate);
			paragraph.appendChild(document.createTextNode("current IP since " + datestring));
			paragraph.appendChild(document.createElement("br"));
			
			// older IPs
			for(var i = 1; i < recent_ips.length; i++) {
				var thisdate = new Date(recent_ips[i]['time'] * 1000);
				
				var ip_link = getClipboardLink(recent_ips[i]['ip']);
				
				paragraph.appendChild(document.createTextNode(formatDate(thisdate) + ": "));
				paragraph.appendChild(ip_link);
				paragraph.appendChild(document.createElement("br"));
			}
			paragraph.appendChild(document.createElement("br"));
		}
		
		// the ping-info
		var span = document.createElement("span");
		span.setAttribute("id", "ping_packets");
		
		// prepare the spans with IDs
		var ping_min = getClipboardLink("?");
		ping_min.setAttribute("id", "ping_min");
		
		var ping_avg = getClipboardLink("?");
		ping_avg.setAttribute("id", "ping_avg");
		
		var ping_max = getClipboardLink("?");
		ping_max.setAttribute("id", "ping_max");
		
		// put it together: the header
		paragraph.appendChild(getDiv("Ping results" + (pingIP ? " for " + pingIP : ''), "title"));
		paragraph.appendChild(span);
		paragraph.appendChild(document.createElement("br"));
		
		// min-ping
		var inline_div = getDiv(0, "inline_div_3");
		inline_div.appendChild(document.createTextNode("Min"));
		inline_div.appendChild(document.createElement("br"));
		inline_div.appendChild(ping_min);
		inline_div.appendChild(document.createTextNode(" ms"));
		paragraph.appendChild(inline_div);
		
		// avg-ping
		var inline_div2 = getDiv(0, "inline_div_3");
		inline_div2.appendChild(document.createTextNode("Avg"));
		inline_div2.appendChild(document.createElement("br"));
		inline_div2.appendChild(ping_avg);
		inline_div2.appendChild(document.createTextNode(" ms"));
		paragraph.appendChild(inline_div2);
		
		// max-ping
		var inline_div3 = getDiv(0, "inline_div_3");
		inline_div3.appendChild(document.createTextNode("Max"));
		inline_div3.appendChild(document.createElement("br"));
		inline_div3.appendChild(ping_max);
		inline_div3.appendChild(document.createTextNode(" ms"));
		paragraph.appendChild(inline_div3);
		
		// dont hide it
		//paragraph.setAttribute("style", "display:block;");
	}
	
	parent.appendChild(paragraph);
	//alert(paragraph.innerHTML + "\n\n");
}


function toClipboard(value) {
	if(window.widget && value) {
		var escaped = escape(value);
		widget.system("./toClipboard.pl '" + escaped + "'", null);
	}
}


// saves the given IP if it is different from the last IP and sends an email (if set in the prefs)
function saveAndNotifyThisIP(ip) {
	//alert("given ip: " + ip + ", recent_ips[0][ip]: " + recent_ips[0]['ip']);
	if(!ip || (recent_ips[0] && (ip == recent_ips[0]['ip'])))
		return;
	
	if(window.widget) {
		
		// notify by email
		if(mail_notification_on) {
			notifyNewIP(ip);
		}
		
		// save new ip
		if(recent_ips.length >= 8) {
			recent_ips.pop();
		}
		var new_arr = new Array();
		new_arr['time'] = Math.round((new Date).getTime() / 1000);
		new_arr['ip'] = ip;
		recent_ips.unshift(new_arr);
		
		// concat all saved ips to a string
		var savestring = '';
		for(var i = 0; i < recent_ips.length; i++) {
			savestring += "|" + recent_ips[i]['time'] + ":" + recent_ips[i]['ip'];
		}
		//alert("saving: " + savestring);
		widget.setPreferenceForKey(savestring, "recentIPs");
	}
}

// sends mail
function notifyNewIP(ip) {
	var to = getNotificationAddresses();
	if(to) {
		concat_addrs = to.join(" ");
		var mails_sent = widget.system("./send_mail.pl " + ip + " " + concat_addrs, null).outputString;
		alert("sent " + mails_sent + " notification mails");
	}
}






/*
 ******************
 * Pref Functions *
 ******************
 */

function getTimer() {		// get refresh-timer from prefs
	if(window.widget) {
		var value = widget.preferenceForKey("timer");
	}
	
	return value ? value : 7200;
}

function getRefreshMode() {		// get mode from prefs (whilehidden, resume)
	if(window.widget) {
		var value = widget.preferenceForKey("refreshMode");
	}
	
	return value ? value : "whilehidden";
}

function getRefreshOnShow() {		// get from prefs
	if(window.widget) {
		var value = widget.preferenceForKey("refreshOnShow");
	}
	
	return value ? true : false;
}

// returns a bool whether to send mails on new IP
function getMailNotificationOn() {
	if(window.widget) {
		var value = widget.preferenceForKey("mail_notification_on");
	}
	
	return value ? true : false;
}

// returns the saved addresses blank
function getNotificationAddressesBlank() {
	if(window.widget) {
		var value = widget.preferenceForKey("notification_mail");
	}
	
	return value ? value : '';
}

// returns an array containing all email-addresses to be notified
function getNotificationAddresses() {
	var value = getNotificationAddressesBlank();
	if(value && value.length > 6)
		value = value.split(/[^\w\.-\+_@]/);
	else
		value = false;
	
	return value ? value : false;
}

function getStylesheet() {		// get from prefs
	if(window.widget) {
		var value = widget.preferenceForKey("stylesheet");
	}
	
	return value ? value : 1;
}


function setStylesheet(style) {
	if(! style)
		style = 1;
	
	var stylelink = document.getElementById('stylesheetlink');
	stylelink.href = "./NetworkStat_" + style + ".css";
	
	// set the correct WS-Logo
	var logo = getObj('infolink').firstChild;
	logo.src = "Images_" + style + "/Amboss.png";
}

function getDateformat() {		// get from prefs
	if(window.widget) {
		var value = widget.preferenceForKey("dateformat");
	}
	
	return ("eu" == value || "us" == value) ? value : "eu";
}

function getPingIP() {		// get from prefs
	if(window.widget) {
		var value = widget.preferenceForKey("pingIP");
	}
	
	return value ? value : "4.2.2.2";
}

// returns a two-dimensional sorted array holding all IPs; returns arr[#][time] and arr[#][ip]
function getRecentIPs() {
	var sorted = new Array();
	
	if(window.widget) {
		//var value = "1145782849:195.186.11.11|1145782591:195.18.2.2|1145682500:127.0.0.1";		// debug
		var value = widget.preferenceForKey("recentIPs");
		var arr = value ? value.split("|") : false;
		
		if(arr) {
			// new array containing the dates
			var sortarr = new Array();
			var fullarr = new Array();
			
			for(var i = 0; i < arr.length; i++) {
				var foo = arr[i].split(":");
				
				if(foo[0] > 0) {
					sortarr.push(foo[0]);
					fullarr[foo[0]] = foo[1];
				}
			}
			
			// sort
			sortarr.sort(NumDESCsort);
			
			// compose to full array
			for(var i = 0; i < sortarr.length; i++) {
				var key = sortarr[i];
				
				if(fullarr[key]) {
					var ar = new Array();
					ar['time'] = key.replace("/\D+/g");
					ar['ip'] = fullarr[key].replace("/[^\.\d]+/g");
					
					sorted.push(ar);
				}
			}
		}
	}
	
	return sorted;
}



// returns the object, whether you supply an id (string) or an object itself
function getObj(id) {
	var obj;
	if(typeof(id) == "string") {
		obj = getById(id) ? getById(id) : null;
	}
	else if(typeof(id) == "object") {
		obj = id;
	}
	
	return obj;
}

// returns the object with the correspondent ID
function getById(id) {
	if(document.getElementById)
		return document.getElementById(id);
	else if(document.all)
		return document.all[id];
	else
		return false;
}


// returns a div
function getDiv(inner, css) {
	var div = document.createElement("div");
	div.setAttribute("class", css ? css : '');
	
	if(inner) {
		var div_inner = ('string' == typeof(inner)) ? document.createTextNode(inner) : inner;
		div.appendChild(div_inner);
	}
	
	return div;
}


// format a clipboard-link
function getClipboardLink(data, css) {
	var link = document.createElement("a");
	
	link.setAttribute("href", "javascript:;");
	link.setAttribute("onclick", "toClipboard(this.innerHTML)");
	link.setAttribute("class", css ? css : "data");
	
	var link_data = document.createTextNode(data);
	link.appendChild(link_data);
	
	return link;
}


// sorts DESC
function NumDESCsort(a, b) {
	a = a.replace(/\D+/g, '');
	b = b.replace(/\D+/g, '');
	
	return b - a;
}



function adjustPrefs() {
	// timer
	var myseconds = getTimer();
	var timerelem = document.forms[0].timer;
	
	if(refreshMode == 'manual') {
		timerelem.options[0].selected = true;
	}
	else {
		selectValue(timerelem, myseconds);
	}
	
	// refresh-mode
	checkIfManually();
	selectValue(document.forms[0].refresh, refreshMode);
	
	// onShow-refresh
	if(refreshOnShow)
		document.forms[0].onshow.checked = true;
	
	// notification
	document.forms[0].mail_notification_on.checked = mail_notification_on ? true : false;
	document.forms[0].notification_mail.value = getNotificationAddressesBlank();
	
	// selected stylesheet
	selectValue(document.forms[0].style, getStylesheet());
	
	// date format
	selectValue(document.forms[0].dateformat, getDateformat());
	
	// insert ping-IP
	document.forms[0].ping_ip.value = pingIP;
}


function checkIfManually() {
	if(document.forms[0].timer.selectedIndex == 0) {
		document.forms[0].refresh.disabled = true;
	}
	else {
		document.forms[0].refresh.disabled = false;
	}
}

function savePrefs() {
	// timer
	var timerelem = document.forms[0].timer;
	var timer = timerelem.options[timerelem.selectedIndex].value;
	timer = (timer == 'manual') ? 7200 : timer;
	
	// mode
	if(timerelem.options[timerelem.selectedIndex].value == 'manual')
		var mode = 'manual';
	else if(document.forms[0].refresh[1].selected == true)
		var mode = "resume";
	else
		var mode = "whilehidden";
	
	// onshow
	refreshOnShow = document.forms[0].onshow.checked;
	
	// notification mail
	mail_notification_on = document.forms[0].mail_notification_on.checked;
	var mailto = document.forms[0].notification_mail.value;
	
	// stylesheet
	var styleform = document.forms[0].style;
	stylesheet = styleform.options[styleform.selectedIndex].value;
	
	// date format
	var dateform = document.forms[0].dateformat;
	dateformat = dateform.options[dateform.selectedIndex].value;
	
	// ping-IP
	pingIP = document.forms[0].ping_ip.value.match(/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/) ? document.forms[0].ping_ip.value : "4.2.2.2";
	
	if(window.widget) {
		widget.setPreferenceForKey(timer, "timer");
		widget.setPreferenceForKey(mode, "refreshMode");
		widget.setPreferenceForKey(refreshOnShow, "refreshOnShow");
		widget.setPreferenceForKey(mail_notification_on, "mail_notification_on");
		widget.setPreferenceForKey(mailto, "notification_mail");
		widget.setPreferenceForKey(stylesheet, "stylesheet");
		widget.setPreferenceForKey(dateformat, "dateformat");
		widget.setPreferenceForKey(pingIP, "pingIP");
	}
	
	hidePrefs();
}

function showPrefs() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	// done-button needs black font instead of white if stylesheet is 3
	var element = document.getElementById("done").firstChild.nextSibling;
	var oldstyle = element.getAttribute("style");
	if(stylesheet == 3)
		element.setAttribute("style", oldstyle + "; color:#000;");
	else
		element.setAttribute("style", oldstyle + "; color:#FFF;");
	
	adjustPrefs();
	
	if(window.widget)
		widget.prepareForTransition("ToBack");
				
	front.style.display = "none";
	back.style.display = "block";
		
	if(window.widget)
		setTimeout("widget.performTransition();", 0);  
}

function switchPrefTab(tab) {
	var found = false;
	var thedivs = new Array("refresh", "mail", "ping", "style");
	
	for(var i = 0; i < thedivs.length; i++) {
		if(thedivs[i] == tab) {
			document.getElementById("tab_" + thedivs[i]).setAttribute("class", "tabitem selected");
			document.getElementById("prefsection_" + thedivs[i]).style.display = "block";
			found = true;
		}
		else {
			document.getElementById("tab_" + thedivs[i]).setAttribute("class", "tabitem");
			document.getElementById("prefsection_" + thedivs[i]).style.display = "none";
		}
	}
	
	if(!found) {
		document.getElementById("prefsection_ping").style.display = "block";
	}
}

function hidePrefs() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	setStylesheet(stylesheet);
	document.getElementById('refreshbutton').src = "Images_" + stylesheet + "/refresh.png";	// set refreshbutton matching stylesheet
	
	seconds = getTimer();
	refreshMode = getRefreshMode();
	refreshOnShow = getRefreshOnShow();
	doCountdown = (refreshMode == 'manual') ? 0 : 1;
	
	if(window.widget)
		widget.prepareForTransition("ToFront");
				
	front.style.display = "block";
	back.style.display = "none";
		
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
	
	setTimeout("refreshState()", 100);
}




/*
 *************
 * Utilities *
 *************
 */

// selects a value in a dropdown list
function selectValue(element, value) {
	if(!element)
		return;
	
	for(i = 0; i < element.length; i++) {
		if(value == element.options[i].value) {
			element.selectedIndex = i;
			return;
		}
	}
}

// checks the preceding Checkbox
function selectVal(clicked) {
	var elem = clicked;
	while(null != elem.previousSibling) {
		if('INPUT' == elem.previousSibling.nodeName) {
			elem.previousSibling.checked = ! elem.previousSibling.checked;
			return;
		}
		elem = elem.previousSibling;
	}
}




//	Localization
/*
function getLocalizedString (key)
{
	try {
		return localizedStrings[key];
	} catch (ex) {}

	return key;
}
*/






/*
 **********
 * Events *
 **********
 */

if(window.widget) {
	widget.onshow = onShow;
	widget.onhide = onHide;
}


function onShow() {
	doCountdown = (refreshMode == 'manual') ? 0 : 1;

	if(refreshOnShow)
		 refreshState();
	
	if(refreshMode == 'resume')		// by now, doCountdown must be 1; else we`re refreshing manually and there needs to be no countdown
		countdown();
	
}

function onHide() {	
	if(refreshMode == 'resume')
		doCountdown = 0;
}





/*
 ********************
 * GUI manipulation *
 ********************
 */


function blendFromTo(from_id, to_id) {
	var from = document.getElementById(from_id);
	var to = document.getElementById(to_id);
	
	if(from && to) {
		from.style.display = 'none';
		to.style.display = 'block';
	}
}


function toggleMoreInfo(obj, event) {
	if(!obj || !obj.nextSibling)
		return;
	
	var moreinfo = obj.nextSibling;
	if(moreinfo == null)
		return;
	
	if(moreinfo.style.display == 'block')
		moreinfo.style.display = 'none';
	else
		moreinfo.style.display = 'block';
	
	adjustTotalSize();
}




var dimensions = new Array();
dimensions[1] = { topheight:54, bottomheight:22, width:250, minheight:140 };
dimensions[2] = { topheight:54, bottomheight:22, width:250, minheight:140 };
dimensions[3] = { topheight:49, bottomheight:18, width:230, minheight:140 };

var resize = { totaltime:300, totallongtime:4000, starttime:null, endtime:null, startsize:0, targetsize:0, windowAdjusted:false };

function adjustTotalSize(longtime) {
	var now = (new Date()).getTime();
	var reference = document.getElementById("maincontent");
	var scale = document.getElementById("main");
	
	resize.startsize = Math.abs(document.defaultView.getComputedStyle(scale).getPropertyValue("height").replace(/px/, ''));
	resize.targetsize = Math.abs(document.defaultView.getComputedStyle(reference).getPropertyValue("height").replace(/px/, ''));
	resize.windowAdjusted = false;
	
	if((resize.startsize < resize.targetsize) && window.widget) {
		var newTotalSize = resize.targetsize + dimensions[stylesheet].topheight + dimensions[stylesheet].bottomheight;
		window.resizeTo(dimensions[stylesheet].width, (newTotalSize < dimensions[stylesheet].minheight) ? dimensions[stylesheet].minheight : newTotalSize); 
		resize.windowAdjusted = true;
	}
	
	resize.starttime = now;
	resize.endtime = now + (longtime ? resize.totallongtime : resize.totaltime);
	
	//alert('resize from ' + resize.startsize + ' to ' + resize.targetsize + ' initiating');
	setTimeout("resizeDiv('main')", 5);
}

function resizeDiv(div) {
	var now = (new Date()).getTime();
	var obj = document.getElementById(div);
	
	// while resizing
	if(now < resize.endtime) {
		var delta = (now - resize.starttime) / resize.totaltime;
		var factor = ((delta * delta) * 3.0) - ((delta * delta * delta) * 2.0);
		var size = (resize.startsize * (1 - factor)) + (resize.targetsize * factor);
		
		obj.style.height = size + "px";
	}
	
	// finishing
	else {
		obj.style.height = resize.targetsize + "px";
		if(!resize.windowAdjusted) {
			var newTotalSize = resize.targetsize + dimensions[stylesheet].topheight + dimensions[stylesheet].bottomheight;
			window.resizeTo(dimensions[stylesheet].width, (newTotalSize < dimensions[stylesheet].minheight) ? dimensions[stylesheet].minheight : newTotalSize);
		}
		return;
	}
	
	setTimeout("resizeDiv('" + div + "')", 10);
}


function formatDate(stamp) {
	if(!stamp)
		stamp = new Date();
	
	var timestring = '';
	
	// date
	var day = stamp.getDate();
	var month = stamp.getMonth() + 1;
	var year = stamp.getFullYear();
	
	// time
	var hours = stamp.getHours();
	hours = (hours < 10) ? "0" + hours : hours;
	var minutes = stamp.getMinutes();
	minutes = (minutes < 10) ? "0" + minutes : minutes;
	var timestring = '';
	
	// compose
	if(dateformat && ("us" == dateformat)) {
		var ampm = "AM";
		if(hours > 12) {
			ampm = "PM";
			hours -= 12;
			hours = (hours < 10) ? "0" + hours : hours;
		}
		timestring = month + "/" + day + "/" + year + " " + hours + ":" + minutes + " " + ampm;
	}
	else {
		timestring = day + "." + month + "." + year + " " + hours + ":" + minutes;
	}
	
	return timestring;
}



/*
 **************
 *  by Apple  *
 **************
 */

var flipShown = false;
var animation = { duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null };

function mousemove(event) {
	if(!flipShown) {
		if (animation.timer != null) {
			clearInterval(animation.timer);
			animation.timer = null;
		}
 
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById('flip');
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}


function mouseexit(event) {
	if(flipShown) {
		
		// fade out the info button
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
	
	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}

function limit_3(a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
	return from + (to - from) * ease;
}


// these functions are called when the info button itself receives onmouseover and onmouseout events (by Apple)

function enterflip(event) {
	document.getElementById("fliprollie").style.display = "block";
}

function exitflip(event) {
	document.getElementById("fliprollie").style.display = "none";
}




